@@ -2,15 +2,12 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-// MSVC++ requires this to be set before any other includes to get M_PI.
-#define _USE_MATH_DEFINES
-
-#include <cmath>
+#include "ui/events/gestures/motion_event_aura.h"
 
 #include "testing/gtest/include/gtest/gtest.h"
 #include "ui/events/event.h"
-#include "ui/events/gestures/motion_event_aura.h"
 #include "ui/events/test/motion_event_test_utils.h"
+#include "ui/gfx/geometry/angle_conversions.h"
 
 namespace {
 
@@ -43,8 +40,8 @@ ui::TouchEvent TouchWithTapParams(ui::EventType type,
   ui::TouchEvent event(
       type, gfx::Point(1, 1), base::TimeTicks(),
       ui::PointerDetails(ui::EventPointerType::POINTER_TYPE_TOUCH, id, radius_x,
-                         radius_y, pressure),
-      0, rotation_angle);
+                         radius_y, pressure, rotation_angle),
+      0);
   return event;
 }
 
@@ -258,9 +255,9 @@ TEST(MotionEventAuraTest, TapParams) {
   EXPECT_EQ(1U, event.GetPointerCount());
   EXPECT_FLOAT_EQ(radius_x, event.GetTouchMajor(0) / 2);
   EXPECT_FLOAT_EQ(radius_y, event.GetTouchMinor(0) / 2);
-  EXPECT_FLOAT_EQ(rotation_angle, event.GetOrientation(0) * 180 / M_PI + 90);
+  EXPECT_FLOAT_EQ(rotation_angle, gfx::RadToDeg(event.GetOrientation(0)) + 90);
   EXPECT_FLOAT_EQ(pressure, event.GetPressure(0));
-  EXPECT_EQ(MotionEvent::TOOL_TYPE_FINGER, event.GetToolType(0));
+  EXPECT_EQ(MotionEvent::ToolType::FINGER, event.GetToolType(0));
 
   // Test case: radius_x < radius_y, rotation_angle < 90
   radius_x = 67.89f;
@@ -274,9 +271,9 @@ TEST(MotionEventAuraTest, TapParams) {
   EXPECT_EQ(2U, event.GetPointerCount());
   EXPECT_FLOAT_EQ(radius_y, event.GetTouchMajor(1) / 2);
   EXPECT_FLOAT_EQ(radius_x, event.GetTouchMinor(1) / 2);
-  EXPECT_FLOAT_EQ(rotation_angle, event.GetOrientation(1) * 180 / M_PI);
+  EXPECT_FLOAT_EQ(rotation_angle, gfx::RadToDeg(event.GetOrientation(1)));
   EXPECT_FLOAT_EQ(pressure, event.GetPressure(1));
-  EXPECT_EQ(MotionEvent::TOOL_TYPE_FINGER, event.GetToolType(1));
+  EXPECT_EQ(MotionEvent::ToolType::FINGER, event.GetToolType(1));
 
   // Test cloning of tap params
   // TODO(mustaq): Make a separate clone test, crbug.com/450655
@@ -286,9 +283,9 @@ TEST(MotionEventAuraTest, TapParams) {
   EXPECT_EQ(2U, clone->GetPointerCount());
   EXPECT_FLOAT_EQ(radius_y, clone->GetTouchMajor(1) / 2);
   EXPECT_FLOAT_EQ(radius_x, clone->GetTouchMinor(1) / 2);
-  EXPECT_FLOAT_EQ(rotation_angle, clone->GetOrientation(1) * 180 / M_PI);
+  EXPECT_FLOAT_EQ(rotation_angle, gfx::RadToDeg(clone->GetOrientation(1)));
   EXPECT_FLOAT_EQ(pressure, clone->GetPressure(1));
-  EXPECT_EQ(MotionEvent::TOOL_TYPE_FINGER, clone->GetToolType(1));
+  EXPECT_EQ(MotionEvent::ToolType::FINGER, clone->GetToolType(1));
 
   // TODO(mustaq): The move test seems out-of-scope here, crbug.com/450655
   radius_x = 76.98f;
@@ -303,9 +300,9 @@ TEST(MotionEventAuraTest, TapParams) {
   EXPECT_EQ(2U, event.GetPointerCount());
   EXPECT_FLOAT_EQ(radius_y, event.GetTouchMajor(1) / 2);
   EXPECT_FLOAT_EQ(radius_x, event.GetTouchMinor(1) / 2);
-  EXPECT_FLOAT_EQ(rotation_angle, event.GetOrientation(1) * 180 / M_PI);
+  EXPECT_FLOAT_EQ(rotation_angle, gfx::RadToDeg(event.GetOrientation(1)));
   EXPECT_FLOAT_EQ(pressure, event.GetPressure(1));
-  EXPECT_EQ(MotionEvent::TOOL_TYPE_FINGER, event.GetToolType(1));
+  EXPECT_EQ(MotionEvent::ToolType::FINGER, event.GetToolType(1));
 
   // Test case: radius_x > radius_y, rotation_angle > 90
   radius_x = 123.45f;
@@ -319,9 +316,9 @@ TEST(MotionEventAuraTest, TapParams) {
   EXPECT_EQ(3U, event.GetPointerCount());
   EXPECT_FLOAT_EQ(radius_x, event.GetTouchMajor(2) / 2);
   EXPECT_FLOAT_EQ(radius_y, event.GetTouchMinor(2) / 2);
-  EXPECT_FLOAT_EQ(rotation_angle, event.GetOrientation(2) * 180 / M_PI + 90);
+  EXPECT_FLOAT_EQ(rotation_angle, gfx::RadToDeg(event.GetOrientation(2)) + 90);
   EXPECT_FLOAT_EQ(pressure, event.GetPressure(2));
-  EXPECT_EQ(MotionEvent::TOOL_TYPE_FINGER, event.GetToolType(2));
+  EXPECT_EQ(MotionEvent::ToolType::FINGER, event.GetToolType(2));
 
   // Test case: radius_x < radius_y, rotation_angle > 90
   radius_x = 67.89f;
@@ -335,9 +332,9 @@ TEST(MotionEventAuraTest, TapParams) {
   EXPECT_EQ(4U, event.GetPointerCount());
   EXPECT_FLOAT_EQ(radius_y, event.GetTouchMajor(3) / 2);
   EXPECT_FLOAT_EQ(radius_x, event.GetTouchMinor(3) / 2);
-  EXPECT_FLOAT_EQ(rotation_angle, event.GetOrientation(3) * 180 / M_PI + 180);
+  EXPECT_FLOAT_EQ(rotation_angle, gfx::RadToDeg(event.GetOrientation(3)) + 180);
   EXPECT_FLOAT_EQ(pressure, event.GetPressure(3));
-  EXPECT_EQ(MotionEvent::TOOL_TYPE_FINGER, event.GetToolType(3));
+  EXPECT_EQ(MotionEvent::ToolType::FINGER, event.GetToolType(3));
 }
 
 TEST(MotionEventAuraTest, Timestamps) {
@@ -374,36 +371,36 @@ TEST(MotionEventAuraTest, CachedAction) {
 
   TouchEvent press0 = TouchWithType(ET_TOUCH_PRESSED, ids[0]);
   EXPECT_TRUE(event.OnTouch(press0));
-  EXPECT_EQ(MotionEvent::ACTION_DOWN, event.GetAction());
+  EXPECT_EQ(MotionEvent::Action::DOWN, event.GetAction());
   EXPECT_EQ(1U, event.GetPointerCount());
 
   TouchEvent press1 = TouchWithType(ET_TOUCH_PRESSED, ids[1]);
   EXPECT_TRUE(event.OnTouch(press1));
-  EXPECT_EQ(MotionEvent::ACTION_POINTER_DOWN, event.GetAction());
+  EXPECT_EQ(MotionEvent::Action::POINTER_DOWN, event.GetAction());
   EXPECT_EQ(1, event.GetActionIndex());
   EXPECT_EQ(2U, event.GetPointerCount());
 
   // Test cloning of CachedAction information.
   std::unique_ptr<MotionEvent> clone = event.Clone();
-  EXPECT_EQ(MotionEvent::ACTION_POINTER_DOWN, clone->GetAction());
+  EXPECT_EQ(MotionEvent::Action::POINTER_DOWN, clone->GetAction());
   EXPECT_EQ(1, clone->GetActionIndex());
 
   TouchEvent move0 = TouchWithType(ET_TOUCH_MOVED, ids[0]);
   move0.set_location(gfx::Point(10, 12));
   EXPECT_TRUE(event.OnTouch(move0));
-  EXPECT_EQ(MotionEvent::ACTION_MOVE, event.GetAction());
+  EXPECT_EQ(MotionEvent::Action::MOVE, event.GetAction());
   EXPECT_EQ(2U, event.GetPointerCount());
 
   TouchEvent release0 = TouchWithType(ET_TOUCH_RELEASED, ids[0]);
   EXPECT_TRUE(event.OnTouch(release0));
-  EXPECT_EQ(MotionEvent::ACTION_POINTER_UP, event.GetAction());
+  EXPECT_EQ(MotionEvent::Action::POINTER_UP, event.GetAction());
   EXPECT_EQ(2U, event.GetPointerCount());
   event.CleanupRemovedTouchPoints(release0);
   EXPECT_EQ(1U, event.GetPointerCount());
 
   TouchEvent release1 = TouchWithType(ET_TOUCH_RELEASED, ids[1]);
   EXPECT_TRUE(event.OnTouch(release1));
-  EXPECT_EQ(MotionEvent::ACTION_UP, event.GetAction());
+  EXPECT_EQ(MotionEvent::Action::UP, event.GetAction());
   EXPECT_EQ(1U, event.GetPointerCount());
   event.CleanupRemovedTouchPoints(release1);
   EXPECT_EQ(0U, event.GetPointerCount());
@@ -415,17 +412,17 @@ TEST(MotionEventAuraTest, Cancel) {
 
   TouchEvent press0 = TouchWithType(ET_TOUCH_PRESSED, ids[0]);
   EXPECT_TRUE(event.OnTouch(press0));
-  EXPECT_EQ(MotionEvent::ACTION_DOWN, event.GetAction());
+  EXPECT_EQ(MotionEvent::Action::DOWN, event.GetAction());
   EXPECT_EQ(1U, event.GetPointerCount());
 
   TouchEvent press1 = TouchWithType(ET_TOUCH_PRESSED, ids[1]);
   EXPECT_TRUE(event.OnTouch(press1));
-  EXPECT_EQ(MotionEvent::ACTION_POINTER_DOWN, event.GetAction());
+  EXPECT_EQ(MotionEvent::Action::POINTER_DOWN, event.GetAction());
   EXPECT_EQ(1, event.GetActionIndex());
   EXPECT_EQ(2U, event.GetPointerCount());
 
   std::unique_ptr<MotionEvent> cancel = event.Cancel();
-  EXPECT_EQ(MotionEvent::ACTION_CANCEL, cancel->GetAction());
+  EXPECT_EQ(MotionEvent::Action::CANCEL, cancel->GetAction());
   EXPECT_EQ(2U, cancel->GetPointerCount());
 }
 
@@ -434,7 +431,7 @@ TEST(MotionEventAuraTest, ToolType) {
   TouchEvent touch_event = TouchWithType(ET_TOUCH_PRESSED, 7);
   EXPECT_TRUE(event.OnTouch(touch_event));
   ASSERT_EQ(1U, event.GetPointerCount());
-  EXPECT_EQ(MotionEvent::TOOL_TYPE_FINGER, event.GetToolType(0));
+  EXPECT_EQ(MotionEvent::ToolType::FINGER, event.GetToolType(0));
 
   touch_event = TouchWithType(ET_TOUCH_RELEASED, 7);
   PointerDetails pointer_details(EventPointerType::POINTER_TYPE_PEN,
@@ -442,9 +439,9 @@ TEST(MotionEventAuraTest, ToolType) {
                                  /* radius_x */ 5.0f,
                                  /* radius_y */ 5.0f,
                                  /* force */ 1.0f);
-  touch_event.set_pointer_details(pointer_details);
+  touch_event.SetPointerDetailsForTest(pointer_details);
   EXPECT_TRUE(event.OnTouch(touch_event));
-  EXPECT_EQ(MotionEvent::TOOL_TYPE_STYLUS, event.GetToolType(0));
+  EXPECT_EQ(MotionEvent::ToolType::STYLUS, event.GetToolType(0));
 }
 
 TEST(MotionEventAuraTest, Flags) {
@@ -504,13 +501,13 @@ TEST(MotionEventAuraTest, UniqueEventID) {
 
   TouchEvent press0 = TouchWithType(ET_TOUCH_PRESSED, 3);
   EXPECT_TRUE(event.OnTouch(press0));
-  EXPECT_EQ(MotionEvent::ACTION_DOWN, event.GetAction());
+  EXPECT_EQ(MotionEvent::Action::DOWN, event.GetAction());
   ASSERT_EQ(1U, event.GetPointerCount());
   EXPECT_EQ(event.GetUniqueEventId(), press0.unique_event_id());
 
   TouchEvent press1 = TouchWithType(ET_TOUCH_PRESSED, 6);
   EXPECT_TRUE(event.OnTouch(press1));
-  EXPECT_EQ(MotionEvent::ACTION_POINTER_DOWN, event.GetAction());
+  EXPECT_EQ(MotionEvent::Action::POINTER_DOWN, event.GetAction());
   EXPECT_EQ(2U, event.GetPointerCount());
   EXPECT_EQ(event.GetUniqueEventId(), press1.unique_event_id());
 }
@@ -540,4 +537,15 @@ TEST(MotionEventAuraTest, IgnoresTouchesOverCapacity) {
   }
 }
 
+TEST(MotionEventAuraTest, PenRadiusDefault) {
+  MotionEventAura event;
+  TouchEvent touch_event = ui::TouchEvent(
+      ET_TOUCH_PRESSED, gfx::Point(0, 0), base::TimeTicks(),
+      ui::PointerDetails(ui::EventPointerType::POINTER_TYPE_PEN, 7));
+  EXPECT_TRUE(event.OnTouch(touch_event));
+  EXPECT_EQ(MotionEvent::ToolType::STYLUS, event.GetToolType(0));
+  EXPECT_EQ(1u, event.GetTouchMajor(0));
+  EXPECT_EQ(1u, event.GetTouchMinor(0));
+}
+
 }  // namespace ui
