@@ -442,6 +442,34 @@ bool QQuickWebEngineSettings::javascriptCanPaste() const
     return d_ptr->testAttribute(WebEngineSettings::JavascriptCanPaste);
 }
 
+/*!
+    \qmlproperty bool WebEngineSettings::dnsPrefetchEnabled
+    \since QtWebEngine 1.8
+
+    Enables speculative prefetching of DNS records for HTML links before
+    they are activated.
+
+    Disabled by default.
+*/
+bool QQuickWebEngineSettings::dnsPrefetchEnabled() const
+{
+    return d_ptr->testAttribute(WebEngineSettings::DnsPrefetchEnabled);
+}
+
+/*!
+    \qmlproperty bool WebEngineSettings::pdfViewerEnabled
+    \since QtWebEngine 1.9
+
+    Specifies that PDF documents will be opened in the internal PDF viewer
+    instead of being downloaded.
+
+    Enabled by default.
+*/
+bool QQuickWebEngineSettings::pdfViewerEnabled() const
+{
+    return d_ptr->testAttribute(WebEngineSettings::PdfViewerEnabled);
+}
+
 /*!
     \qmlproperty string WebEngineSettings::defaultTextEncoding
     \since QtWebEngine 1.2
@@ -692,6 +720,22 @@ void QQuickWebEngineSettings::setJavascriptCanPaste(bool on)
         Q_EMIT javascriptCanPasteChanged();
 }
 
+void QQuickWebEngineSettings::setDnsPrefetchEnabled(bool on)
+{
+    bool wasOn = d_ptr->testAttribute(WebEngineSettings::DnsPrefetchEnabled);
+    d_ptr->setAttribute(WebEngineSettings::DnsPrefetchEnabled, on);
+    if (wasOn != on)
+        Q_EMIT dnsPrefetchEnabledChanged();
+}
+
+void QQuickWebEngineSettings::setPdfViewerEnabled(bool on)
+{
+    bool wasOn = d_ptr->testAttribute(WebEngineSettings::PdfViewerEnabled);
+    d_ptr->setAttribute(WebEngineSettings::PdfViewerEnabled, on);
+    if (wasOn != on)
+        Q_EMIT pdfViewerEnabledChanged();
+}
+
 void QQuickWebEngineSettings::setUnknownUrlSchemePolicy(QQuickWebEngineSettings::UnknownUrlSchemePolicy policy)
 {
     WebEngineSettings::UnknownUrlSchemePolicy oldPolicy = d_ptr->unknownUrlSchemePolicy();
