@@ -219,6 +219,7 @@ RenderWidgetHostViewQtDelegate *QQuickWebEngineViewPrivate::CreateRenderWidgetHo
     RenderWidgetHostViewQtDelegateQuick *quickDelegate = new RenderWidgetHostViewQtDelegateQuick(client, /*isPopup = */ true);
     if (hasWindowCapability) {
         RenderWidgetHostViewQtDelegateQuickWindow *wrapperWindow = new RenderWidgetHostViewQtDelegateQuickWindow(quickDelegate);
+        wrapperWindow->setVirtualParent(q);
         quickDelegate->setParentItem(wrapperWindow->contentItem());
         return wrapperWindow;
     }
@@ -941,6 +942,16 @@ void QQuickWebEngineViewPrivate::updateAction(QQuickWebEngineView::WebAction act
     case QQuickWebEngineView::ViewSource:
         enabled = adapter->canViewSource();
         break;
+    case QQuickWebEngineView::Cut:
+    case QQuickWebEngineView::Copy:
+    case QQuickWebEngineView::Paste:
+    case QQuickWebEngineView::Undo:
+    case QQuickWebEngineView::Redo:
+    case QQuickWebEngineView::SelectAll:
+    case QQuickWebEngineView::PasteAndMatchStyle:
+    case QQuickWebEngineView::Unselect:
+        enabled = adapter->hasFocusedFrame();
+        break;
     default:
         break;
     }
@@ -958,6 +969,18 @@ void QQuickWebEngineViewPrivate::updateNavigationActions()
     updateAction(QQuickWebEngineView::ViewSource);
 }
 
+void QQuickWebEngineViewPrivate::updateEditActions()
+{
+    updateAction(QQuickWebEngineView::Cut);
+    updateAction(QQuickWebEngineView::Copy);
+    updateAction(QQuickWebEngineView::Paste);
+    updateAction(QQuickWebEngineView::Undo);
+    updateAction(QQuickWebEngineView::Redo);
+    updateAction(QQuickWebEngineView::SelectAll);
+    updateAction(QQuickWebEngineView::PasteAndMatchStyle);
+    updateAction(QQuickWebEngineView::Unselect);
+}
+
 QUrl QQuickWebEngineView::url() const
 {
     Q_D(const QQuickWebEngineView);
